#ifndef W5500_H
#define W5500_H

#define W5500_OPERATION_FAIL		0
#define W5500_OPERATION_PASS		1

#define OFFSET_CHIP_MODE_ADDRESS					0x0000		//0x0000
#define OFFSET_CHIP_GATEWAY_ADDRESS					0x0001    	//0x0001 - 0x0004
#define OFFSET_CHIP_SUBNET_ADDRESS					0x0005		//0x0005 - 0x0008
#define OFFSET_CHIP_MAC_ADDRESS						0x0009		//0x0009 - 0x000C
#define OFFSET_CHIP_IP_ADDRESS						0x000F   	//0x000F - 0x0012
#define OFFSET_CHIP_INT_TIMER_VALUE_ADDRESS 		0x0013		//0x0013 - 0x0014
#define OFFSET_CHIP_INT_ADDRESS						0x0015		//0x0015
#define OFFSET_CHIP_INT_MASK_ADDRESS				0x0016		//0x0016
#define OFFSET_CHIP_SOCK_INT_ADDRESS				0x0017		//0x0017
#define OFFSET_CHIP_SOCK_INT_MASK_ADDRESS			0x0018		//0x0018
#define OFFSET_CHIP_INT_TIMER_RETRANSMIT_ADDRESS	0x0019		//0x0019-0x001A
#define OFFSET_CHIP_RETRANSMIT_COUNT_ADDRESS		0x001B		//0x001B
#define OFFSET_CHIP_PHYSICAL_LINK_ADDRESS			0x002E		//0x002E
#define OFFSET_CHIP_VERSION							0x0039		//0x0039


#define OFFSET_SOCKET_MODE_ADDRESS					0x0000		//0x0000
#define OFFSET_SOCKET_COMMAND_ADDRESS				0x0001		//0x0001
#define OFFSET_SOCKET_INTERRUPT_ADDRESS				0x0002		//0x0002
#define OFFSET_SOCKET_STATUS_ADDRESS				0x0003		//0x0003
#define OFFSET_SOCKET_SOURCE_PORT_ADDRESS			0x0004		//0x0004 - 0x0005
#define OFFSET_SOCKET_DEST_HW_ADDRESS				0x0006		//0x0006 - 0x000B
#define OFFSET_SOCKET_DESTINATION_IP_ADDRESS		0x000C		//0x000C - 0x000F
#define OFFSET_SOCKET_DESTINATION_PORT_ADDRESS		0x0010 		//0x0010 - 0x0011
#define OFFSET_SOCKET_RX_MAX_SIZE_ADDRESS			0x001E		//0x001E
#define OFFSET_SOCKET_TX_MAX_SIZE_ADDRESS			0x001F		//0x001F
#define OFFSET_SOCKET_TX_FREE_SIZE_BUFFER_ADDRESS	0x0020		//0x0020 - 0x0021
#define OFFSET_SOCKET_TX_READ_POINTER_ADDRESS		0x0022		//0x0022 - 0x0023
#define OFFSET_SOCKET_TX_WRITE_POINTER_ADDRESS		0x0024		//0x0024 - 0x0025

#define OFFSET_SOCKET_RX_RECV_SIZE_ADDRESS			0x0026		//0x0026 - 0x0027
#define OFFSET_SOCKET_RX_READ_POINTER_ADDRESS		0x0028		//0x0028 - 0x0029
#define OFFSET_SOCKET_RX_WRITE_POINTER_ADDRESS		0x002A   	//0x002A - 0x002B

#define OFFSET_SOCKET_INT_MASK_ADDRESS				0x002C		//0x002C
#define OFFSET_SOCKET_KEEP_ALIVE_ADDRESS			0x002F		//0x002F


#define SOCKET_WRITE_OPERATION			0x04
#define SOCKET_READ_OPERATION			0x00

#define BSB_SOCKET0_REGISTER_BASE				8
#define BSB_SOCKET0_TX_BUFFER_BASE				16
#define BSB_SOCKET0_RX_BUFFER_BASE				24
#define BSB_SOCKET0_RESERVED_BASE				32


struct W5500_NETWORK
{

	unsigned char m_ucIp[4];
	unsigned char m_ucSubnetMask[4];
	unsigned char m_ucGateway[4];
	unsigned char m_ucMAC[6];

	unsigned short m_usSrcPort;
	unsigned char m_ucEnSocInt;
	unsigned char m_ucEnSocMask;
	unsigned short m_usRetryTime;
	unsigned char m_ucRetryCount;
};
extern volatile struct W5500_NETWORK W5500_Network;





void W5500_Init(void);
void W5500_HW_Reset(void);

unsigned char W5500_SetMode(unsigned char mode);
unsigned char W5500_GetMode(unsigned char *mode);

unsigned char W5500_SetIp(unsigned char*);
unsigned char W5500_GetIp(unsigned char*);

unsigned char W5500_SetGateway(unsigned char*);
unsigned char W5500_GetGateway(unsigned char*);


unsigned char W5500_SetMac(unsigned char*);
unsigned char W5500_GetMac(unsigned char*);


unsigned char W5500_SetSubnetMask(unsigned char*);
unsigned char W5500_GetSubnetMask(unsigned char*);

unsigned char W5500_SetNetwork(volatile struct W5500_NETWORK*);
unsigned char W5500_GetNetwork(volatile struct W5500_NETWORK*);


unsigned char W5500_SetIntTimer(unsigned short);
unsigned char W5500_GetIntTimer(unsigned short*);

unsigned char W5500_SetIntReg(unsigned char);
unsigned char W5500_GetIntReg(unsigned char*);

unsigned char W5500_SetIntMaskReg(unsigned char);
unsigned char W5500_GetIntMaskReg(unsigned char*);


unsigned char W5500_SetSockIntReg(unsigned char);
unsigned char W5500_GetSockIntReg(unsigned char*);

unsigned char W5500_SetSockIntMaskReg(unsigned char);
unsigned char W5500_GetSockIntMaskReg(unsigned char*);

unsigned char W5500_SetReTransmitTime(unsigned short);
unsigned char W5500_GetReTransmitTime(unsigned short*);

unsigned char W5500_SetReTransmitCount(unsigned char);
unsigned char W5500_GetReTransmitCount(unsigned char*);

unsigned char W5500_GetPhyConfig(unsigned char*);



unsigned char W5500_GetChipVersio(unsigned char*);

void W5500_NetworkInit(unsigned char*,unsigned char*,unsigned char*,unsigned char*);



//Socket

unsigned char W5500_Set_Socket_Mode(unsigned char,unsigned char);
unsigned char W5500_Get_Socket_Mode(unsigned char,unsigned char*);

unsigned char W5500_Set_CommandReg(unsigned char,unsigned char);
unsigned char W5500_Get_CommandReg(unsigned char,unsigned char*);

unsigned char W5500_Set_InterruptReg(unsigned char,unsigned char);
unsigned char W5500_Get_InterruptReg(unsigned char,unsigned char*);

unsigned char W5500_Get_StatusReg(unsigned char,unsigned char*);


unsigned char W5500_Set_SourcePort(unsigned char,unsigned short);
unsigned char W5500_Get_SourcePort(unsigned char,unsigned short*);

unsigned char W5500_Set_Dest_HW_Addr(unsigned char,unsigned char*);
unsigned char W5500_Get_Dest_HW_Addr(unsigned char,unsigned char*);

unsigned char W5500_Set_Dest_Ip(unsigned char,unsigned char*);
unsigned char W5500_Get_Dest_Ip(unsigned char,unsigned char*);

unsigned char W5500_Set_Dest_Port(unsigned char,unsigned short);
unsigned char W5500_Get_Dest_Port(unsigned char,unsigned short*);



unsigned char W5500_Get_Tx_FreeSize(unsigned char,unsigned short*);

unsigned char W5500_Get_Tx_ReadPointer(unsigned char,unsigned short*);

unsigned char W5500_Set_Tx_WritePointer(unsigned char,unsigned short);
unsigned char W5500_Get_Tx_WritePointer(unsigned char,unsigned short*);

unsigned char W5500_Get_Rx_Recv_Size(unsigned char,unsigned short*);

unsigned char W5500_Set_Rx_ReadPointer(unsigned char,unsigned short);
unsigned char W5500_Get_Rx_ReadPointer(unsigned char,unsigned short*);

unsigned char W5500_Get_Rx_WritePointer(unsigned char,unsigned short*);


unsigned char W5500_Set_IntMask(unsigned char,unsigned char);
unsigned char W5500_Get_IntMask(unsigned char,unsigned char*);


unsigned char W5500_SET_TxBuffer(unsigned char,unsigned char);
unsigned char W5500_GET_TxBuffer(unsigned char,unsigned char*);

unsigned char W5500_SET_RxBuffer(unsigned char,unsigned char);
unsigned char W5500_GET_RxBuffer(unsigned char,unsigned char*);



unsigned char W5500_Set_KeepAlive(unsigned char,unsigned char);






unsigned char W5500_Write_Data_Tx_Buffer(unsigned char,unsigned short,unsigned char*,unsigned short);
unsigned char W5500_Read_Data_Rx_Buffer(unsigned char,unsigned short,unsigned char*,unsigned short);


#endif
